import Flutter
import UIKit
import BankConnect

public class FinBoxBcPlugin: NSObject, FlutterPlugin {
    
    // Flutter method channel
    static var channel: FlutterMethodChannel? = nil
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        channel = FlutterMethodChannel(name: "in.finbox.bankconnect/hybrid", binaryMessenger: registrar.messenger())
        let instance = FinBoxBcPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel!)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "initSdk":
            guard let args = call.arguments else {
                print("No arguments are returned")
                return
            }
            if let myArgs = args as? [String: Any],
               let apiKey = myArgs["apiKey"] as? String,
               let fromDate = myArgs["fromDate"] as? String,
               let toDate = myArgs["toDate"] as? String,
               let bankName = myArgs["bankName"] as? String {
                let _ = FinBoxBankConnect.Builder()
                    .linkId(id: UUID().uuidString)
                    .apiKey(key: apiKey)
                    .fromDate(start:fromDate)
                    .toDate(end:toDate)
                    .bank(name:bankName)
                    .build();
                
                // Create a View Controller
                let vc = FinBoxViewController()
                
                // Get access to the window
                let keyWindow = UIApplication
                    .shared
                    .connectedScenes
                    .flatMap { ($0 as? UIWindowScene)?.windows ?? [] }
                    .last { $0.isKeyWindow }
                
                // Get the view controller at root
                let viewController = keyWindow?.rootViewController
                
                // Attach the view controller to the root
                let nav = UINavigationController(rootViewController: vc)
                
                nav.modalPresentationStyle = .overFullScreen
                
                // Show the view on the screen
                viewController?.present(nav, animated: true,completion: nil)
            }
            
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
}
